/*
 * Menu.h
 *
 *  Created on: Dec 10, 2009
 *      Author: crueger
 */

#ifndef MENU_MENU_H_
#define MENU_MENU_H_

#include <set>
#include <string>

#include "Menu/MenuInterface.hpp"
#include "Menu/MenuDescription.hpp"

/** Base class for all Types of menus.
 * Here, we simply initialize the menus. Via the MenuInterface wrapper we may
 * access the adding of items for each specific menu in a uniform manner.
 *
 * Note that this Class is never to be used directly but only via derived
 * specializations.
 */
class Menu : virtual public MenuInterface
{
public:
  explicit Menu(const std::string &name);
  virtual ~Menu();

  void init();

protected:
  // Unique name of the menu for identification.
  const std::string name;

  // populater function that adds all menu items
  void populate();
  void populateActions();

private:
  void addAction(const std::string &ActionName);
  void addSeparator();
  void addSubmenu(const std::string &MenuName, const int MenuPosition);
  bool isPresent(const std::string &token);

  enum ItemType {ActionItem, MenuItem, SeparatorItem, NoItem};

  int TopPosition;  // current position to add
  MenuDescription menudescriptions; // contains the menu tree and description to each item
  enum ItemType LastItem;  // check whether separator followed separator
  std::set <std::string> DuplicatesList;  // is used to check for duplicates
};

#endif /* MENU_H_ */
